#include "PStream.h"
#include <Bios.h>

prnbuf::prnbuf()
	{
	xfd = EOF;
	isBinary = 0;
	char* p = new char[B_size];
	if( p )
		{
		setb( p, p+B_size,1 );	// ~streambuf() will delete.
		setp(base(),ebuf());
		setg(0,0,0);
		}
	}

prnbuf::prnbuf(int f)
	{
	xfd = EOF;
	isBinary = 0;

	// Check that port is supported.

	unsigned u = ((unsigned)(biosequip()) >> 14) & 0x0003;
	// u is the number of printers supported by BIOS.

	if( f >= 0 && f < u )
		{

		// Make sure printer is there.

		unsigned u = biosprint( _PRINTER_STATUS, 0, f );
		if( (u & pfSelected) == 0 )
			return;
		xfd = f;
		}

	char* p = new char[B_size];
	if( p )
		{
		setb(p, p+B_size, 1);	// ~streambuf() will delete.
		setp(base(),ebuf());
		setg(0,0,0);
		}
	}

prnbuf::prnbuf(int f, char* buf, int len)
	{
	xfd = EOF;
	isBinary = 0;

	// Check that port is supported.

	unsigned u = ((unsigned)(biosequip()) >> 14) & 0x0003;
	if( f >= 0 && f < u )
		{

		// Make sure printer is there.

		unsigned u = biosprint( _PRINTER_STATUS, 0, f );
		if( (u & pfSelected) == 0 )
			return;
		xfd = f;
		}

	setbuf(buf, len);
	setp(base(),ebuf());
	setg(0,0,0);
	}

prnbuf::~prnbuf()
	{
	if( xfd >= 0 )
		sync();		// flush the buffer.
	}

void prnbuf::init()
	{
	if( xfd < 0 )
		return;
	biosprint( _PRINTER_INIT, 0, xfd );
	}

prnbuf* prnbuf::attach(int f)
	{
	// See if supported by equipment list.

	unsigned u = ((unsigned)(biosequip()) >> 14) & 0x0003;
	if( f >= 0 && f < u )
		{

		// Make sure printer is there.

		unsigned u = biosprint( _PRINTER_STATUS, 0, f );
		if( (u & pfSelected) == 0 )
			return 0;
		}
	else
		return 0;

	// If already attached to a port, flush.

	if( xfd >= 0 && sync() == EOF )
		return 0;	// do NOT un-attach if flush fails.

	xfd = f;        // assumed to be valid (IE on, not out of paper...)
	isBinary = 0;
	char *b = base();       // buffer address
	if( ! b )
		{
		b = new char[B_size];
		if( b )
			{
			setb( b, b+B_size, 1 );	// ~streambuf() will delete.
			setp(base(),base());
			setg(0,0,0);
			}
		}
	return this;
	}

int prnbuf::overflow(int c)
	{
	unsigned st;

	if( xfd < 0 )
		return EOF;

	if( unbuffered() || !base() )
		{
		if( c != EOF )
			{
			char b = c;
			st = biosprint( _PRINTER_WRITE, b, xfd );
			if( (st & (pfOutOfPaper | pfIoError | pfTimeout)) != 0 )
				return EOF;

			if( b == '\n' && !isBinary )
				{
				st = biosprint( _PRINTER_WRITE, '\r', xfd );
				if( (st & (pfOutOfPaper | pfIoError | pfTimeout)) != 0 )
					return EOF;
				}
			}
		}
	else	// flush.
		{
		char *p = pbase();
		char *ep = pptr();
		while( p < ep )
			{
			st = biosprint( _PRINTER_WRITE, *p, xfd );
			if( (st & (pfOutOfPaper | pfIoError | pfTimeout)) != 0 )
				{
				setp( p, ebuf() );
				return EOF;
				}
			if( *p == '\n' && !isBinary )
				{
				st = biosprint( _PRINTER_WRITE, '\r', xfd );
				if( (st & (pfOutOfPaper | pfIoError | pfTimeout)) != 0 )
					return EOF;
				}
			p++;
			}
		setp( base(), ebuf() );
		if( c != EOF )
			sputc(c);
		}
	return 1;
	}

int prnbuf::underflow()
	{
	if( xfd < 0 )
		return EOF;

	return biosprint( _PRINTER_STATUS, 0, xfd );
	}

int prnbuf::sync()
	{
	if( xfd < 0 )	return EOF;
	return overflow(EOF);
	}


prnstreambase::prnstreambase()
		: buf()
	{
	ios::init(&buf);
	}

prnstreambase::prnstreambase(int f)
		: buf(f)
	{
	ios::init(&buf);
	if( buf.fd() < 0 )
		clear(ios::badbit);
	else
		clear(ios::goodbit);
	}

prnstreambase::prnstreambase(int f, char* b, int len)
		: buf(f, b, len)
	{
	ios::init(&buf);
	if( buf.fd() < 0 )
		clear(ios::badbit);
	else
		clear(ios::goodbit);
	}

prnstreambase::~prnstreambase()
	{
	}

void prnstreambase::attach(int f)
	{
	if( buf.attach(f) )
		clear(ios::goodbit);
	else
		clear(ios::badbit);
	}

void prnstreambase::setbuf(char* b, int len)
	{
	if( buf.setbuf(b, len) )
		clear(ios::goodbit);
	else
		setstate(ios::failbit);
	}

prnstream::prnstream() :
		prnstreambase(),
		ostream()
	{
	}

prnstream::prnstream(int f) :
		prnstreambase(f),
		ostream()
	{
	}

prnstream::prnstream(int f, char* b, int len) :
		prnstreambase(f, b, len),
		ostream()
	{
	}

prnstream::~prnstream()
	{
	}

